/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.geoviewer.jxmap.map.cache.spi;

import java.awt.image.BufferedImage;
import nl.cloudfarming.client.geoviewer.jxmap.map.cache.TileCacheInfo;

/**
 *
 * @author Frantisek Post
 */
public class DefaultTileService extends AbstractTileService {

    private static final int HITS_TO_CLEAN_CACHE = 30; //every 30th write to disk trigger clean of cache
    private int hitCounter = 0;
    private static DefaultTileService instance = new DefaultTileService();
    
    private DefaultTileService() {
    }
    
    @Override
    protected void writeCallback() {
        super.writeCallback();
        if (!CacheCleaner.getInstance().isUnlimited()) {
            hitCounter++;
            if (hitCounter >= HITS_TO_CLEAN_CACHE) {
                hitCounter = 0;
                CacheCleaner.getInstance().run();
            }
        }
    }
    
    //API
    
    @Override
    public boolean contains(TileCacheInfo tileCacheInfo) {
        return CacheManager.getInstance().isCached(tileCacheInfo.getZoom(), tileCacheInfo.getX(), tileCacheInfo.getY());
    }

    @Override
    public BufferedImage getTile(TileCacheInfo tileCacheInfo) {
        return readImage(tileCacheInfo);
    }
    
    public void writeTile(TileCacheInfo tileCacheInfo, BufferedImage image) {
        writeImage(tileCacheInfo, image);
    }
    
    public static DefaultTileService getInstance() {
        return instance;
    }

}
